// $Id: HotkeyObject.cpp,v 1.4 2000/02/16 05:56:41 mian Exp $

#include "stdafx.h"
#include "Core.h"
#include "HotkeyObject.h"
#include "../common/gdefs.h"
#include "../common/util_safe.h"
#include "../common/util_strings.h"

static TCHAR lpszHotkeyClass[] = L"GraphiteHotkeyClass";

class CHotKey
{
public:
	BSTR m_bstrName;
	USHORT m_uKey;
};

static CHotKey g_hotkeys[] = {
	{ L"A",			'A'	},
	{ L"B",			'B'	},
	{ L"C",			'C'	},
	{ L"D",			'D'	},
	{ L"E",			'E'	},
	{ L"F",			'F'	},
	{ L"G",			'G'	},
	{ L"H",			'H'	},
	{ L"I",			'I'	},
	{ L"J",			'J'	},
	{ L"K",			'K'	},
	{ L"L",			'L'	},
	{ L"M",			'M'	},
	{ L"N",			'N'	},
	{ L"O",			'O'	},
	{ L"P",			'P'	},
	{ L"Q",			'Q'	},
	{ L"R",			'R'	},
	{ L"S",			'S'	},
	{ L"T",			'T'	},
	{ L"U",			'U'	},
	{ L"V",			'V'	},
	{ L"W",			'W'	},
	{ L"X",			'X'	},
	{ L"Y",			'Y'	},
	{ L"Z",			'Z'	},
	{ L"0",			'0'	},
	{ L"1",			'1'	},
	{ L"2",			'2'	},
	{ L"3",			'3'	},
	{ L"4",			'4'	},
	{ L"5",			'5'	},
	{ L"6",			'6'	},
	{ L"7",			'7'	},
	{ L"8",			'8'	},
	{ L"9",			'9'	},
	{ L"SPACE",		0x20 },
	{ L"PRIOR",		0x21 },
	{ L"NEXT",		0x22 },
	{ L"END",		0x23 },
	{ L"HOME",		0x24 },
	{ L"LEFT",		0x25 },
	{ L"UP",		0x26 },
	{ L"RIGHT",		0x27 },
	{ L"DOWN",		0x28 },
	{ L"SELECT",	0x29 },
	{ L"PRINT",		0x2A },
	{ L"EXECUTE",	0x2B },
	{ L"SNAPSHOT",	0x2C },
	{ L"INSERT",	0x2D },
	{ L"DELETE",	0x2E },
	{ L"HELP",		0x2F },
	{ L"NUMPAD0",	0x60 },
	{ L"NUMPAD1",	0x61 },
	{ L"NUMPAD2",	0x62 },
	{ L"NUMPAD3",	0x63 },
	{ L"NUMPAD4",	0x64 },
	{ L"NUMPAD5",	0x65 },
	{ L"NUMPAD6",	0x66 },
	{ L"NUMPAD7",	0x67 },
	{ L"NUMPAD8",	0x68 },
	{ L"NUMPAD9",	0x69 },
	{ L"MULTIPLY",	0x6A },
	{ L"ADD",		0x6B },
	{ L"SEPARATOR",	0x6C },
	{ L"SUBTRACT",	0x6D },
	{ L"DECIMAL",	0x6E },
	{ L"DIVIDE",	0x6F },
	{ L"F1",		0x70 },
	{ L"F2",		0x71 },
	{ L"F3",		0x72 },
	{ L"F4",		0x73 },
	{ L"F5",		0x74 },
	{ L"F6",		0x75 },
	{ L"F7",		0x76 },
	{ L"F8",		0x77 },
	{ L"F9",		0x78 },
	{ L"F10",		0x79 },
	{ L"F11",		0x7A },
	{ L"F12",		0x7B },
	{ L"F13",		0x7C },
	{ L"F14",		0x7D },
	{ L"F15",		0x7E },
	{ L"F16",		0x7F },
	{ L"F17",		0x80 },
	{ L"F18",		0x81 },
	{ L"F19",		0x82 },
	{ L"F20",		0x83 },
	{ L"F21",		0x84 },
	{ L"F22",		0x85 },
	{ L"F23",		0x86 },
	{ L"F24",		0x87 },
	{ L"NUMLOCK",	0x90 },
	{ L"SCROLL",	0x91 }
};

/////////////////////////////////////////////////////////////////////////////
// CHotkeyObject

STDMETHODIMP CHotkeyObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IHotkeyObject,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CHotkeyObject::CHotkeyObject()
{
	m_hWnd = NULL;

	/*
		Register our classes and create the hotkey handler window
	*/
	WNDCLASS wc;
	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = CHotkeyObject::ProxyWndProc;
	wc.hInstance = GetModuleHandle(NULL);
	wc.lpszClassName = lpszHotkeyClass;
	wc.style = CS_DBLCLKS;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.cbWndExtra = 4;
	if (!RegisterClass(&wc))
	{
		return;
	}

	HWND hWnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		lpszHotkeyClass,
		NULL,
		WS_POPUP,
		0, 0,
		0, 0,
		NULL,
		NULL,
		GetModuleHandle(NULL),
		NULL
		);

	if (hWnd)
	{
		SetWindowLong(hWnd, G_GWL_CLASSPOINTER, (LONG)this);
		ShowWindow(hWnd, SW_HIDE);
	}

	m_hWnd = hWnd;
}

CHotkeyObject::~CHotkeyObject()
{
	if (m_hWnd)
	{
		DestroyWindow(m_hWnd);
		m_hWnd = NULL;
	}
	UnregisterClass(lpszHotkeyClass, GetModuleHandle(NULL));
}

LRESULT WINAPI CHotkeyObject::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		/*
			Trigger the command in the scripting host
		*/
		case WM_HOTKEY: Fire_OnCommand(wParam); break;

		default:
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
	return 0;
}

LRESULT WINAPI CHotkeyObject::ProxyWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CHotkeyObject *p = (CHotkeyObject *)GetWindowLong(hWnd, G_GWL_CLASSPOINTER);
	if (IsValidReadPtr(p)) return p->WndProc(hWnd, uMsg, wParam, lParam);
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


STDMETHODIMP CHotkeyObject::Add(BSTR bstrModifiers, BSTR bstrKey, long uID)
{
	UINT uModifiers = 0;
	UINT uKey = 0;
	int i;

	/*
		See if the user specified a key name for special keys in the table above
	*/
	for (i = 0; i < sizeof(g_hotkeys) / sizeof(CHotKey); i++)
	{
		if (!_wcsicmp(g_hotkeys[i].m_bstrName, bstrKey))
			uKey = g_hotkeys[i].m_uKey;
	}

	CAStr asz(bstrModifiers);
	CHAR *lpszPtr;
	lpszPtr = strtok(asz.m_sz, "+");
	do 
	{
		if (lpszPtr)
		{
			if (!stricmp(lpszPtr, "ALT"))
				uModifiers |= MOD_ALT;
			else if (!stricmp(lpszPtr, "WIN"))
				uModifiers |= MOD_WIN;
			else if (!stricmp(lpszPtr, "CTRL"))
				uModifiers |= MOD_CONTROL;
			else if (!stricmp(lpszPtr, "SHIFT"))
				uModifiers |= MOD_SHIFT;
			lpszPtr = strtok(NULL, "+");
		}
	} while (lpszPtr);
	RegisterHotKey(m_hWnd, uID, uModifiers, uKey);

	return S_OK;
}

STDMETHODIMP CHotkeyObject::Remove(long uID)
{
	UnregisterHotKey(m_hWnd, uID);

	return S_OK;
}

/*
	$Log: HotkeyObject.cpp,v $
	Revision 1.4  2000/02/16 05:56:41  mian
	
	Cleaned up class pointer code
	
	Revision 1.3  2000/02/13 08:04:07  mian
	
	Added log messages
	
*/