// Ripple.h : Declaration of the CRipple

#ifndef __RIPPLE_H_
#define __RIPPLE_H_

#include "resource.h"       // main symbols
#include <atlctl.h>
#include <math.h>
#include "../common/util_list.h"
#include "../common/util_safe.h"

class CRect
{
public:
	int left;
	int top;
	int right;
	int bottom;

	CRect()
	{
		left = 0;
		top = 0;
		right = 0;
		bottom = 0;
	}
};

/////////////////////////////////////////////////////////////////////////////
// CRipple
class ATL_NO_VTABLE CRipple : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDispatchImpl<IRipple, &IID_IRipple, &LIBID_DESKEFXLib>,
	public CComControl<CRipple>,
	public IPersistStreamInitImpl<CRipple>,
	public IOleControlImpl<CRipple>,
	public IOleObjectImpl<CRipple>,
	public IOleInPlaceActiveObjectImpl<CRipple>,
	public IViewObjectExImpl<CRipple>,
	public IOleInPlaceObjectWindowlessImpl<CRipple>,
	public ISupportErrorInfo,
	public IPersistStorageImpl<CRipple>,
	public ISpecifyPropertyPagesImpl<CRipple>,
	public IQuickActivateImpl<CRipple>,
	public IDataObjectImpl<CRipple>,
	public IProvideClassInfo2Impl<&CLSID_Ripple, NULL, &LIBID_DESKEFXLib>,
	public CComCoClass<CRipple, &CLSID_Ripple>
{
public:
	CRipple()
	{
		m_bWindowOnly = TRUE;

		m_nIncv = 0;
		m_nInch = 0;
		m_nWidth = 0;
		m_nHeight = 0;
		m_hbmImage = NULL;
		m_hImageDC = NULL;
	}

	virtual ~CRipple()
	{
		if (m_hImageDC) DeleteDC(m_hImageDC);
		if (m_hbmImage) DeleteObject(m_hbmImage);
		m_rectRipples.DeleteItems();
	}

DECLARE_REGISTRY_RESOURCEID(IDR_RIPPLE)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CRipple)
	COM_INTERFACE_ENTRY(IRipple)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(CRipple)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CRipple)
	CHAIN_MSG_MAP(CComControl<CRipple>)
	DEFAULT_REFLECTION_HANDLER()
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MESSAGE_HANDLER(WM_TIMER, OnTimer)
	MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBkgnd)
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
	{
		static const IID* arr[] = 
		{
			&IID_IRipple,
		};
		for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
		{
			if (InlineIsEqualGUID(*arr[i], riid))
				return S_OK;
		}
		return S_FALSE;
	}

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IRipple
public:
	STDMETHOD(ClearRippleAreas)();
	STDMETHOD(AddRippleArea)(long left, long top, long right, long bottom);
	STDMETHOD(get_ImageHeight)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_ImageWidth)(/*[out, retval]*/ long *pVal);
	STDMETHOD(LoadImage)(BSTR bstrPath);
	BOOL BufferImage(BSTR lpszPath);

	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di)
	{
		if (m_hbmImage && m_hImageDC)
		{
			HBITMAP hbmBuf;
			HDC hBufDC;

			hBufDC = CreateCompatibleDC(di.hdcDraw);
			hbmBuf = CreateCompatibleBitmap(di.hdcDraw, m_nWidth, m_nHeight);
			SelectObject(hBufDC, hbmBuf);

			BitBlt(hBufDC, 0, 0, m_nWidth, m_nHeight, m_hImageDC, 0, 0, SRCCOPY);

			m_nIncv += 0.09;
			if (m_nIncv > (M_PI_2*4))
			{
				m_nIncv = 0;
			}
			m_nInch += 0.06;
			if (m_nInch > (M_PI_2*4))
			{
				m_nInch = 0;
			}

			int waterh, waterw;
			int y;
			int off;
			int yoff;
			double a;
			double p;
			unsigned int i;
			CRect *pRect;

			for (i = 0; i < m_rectRipples.GetSize(); i++)
			{
				pRect = m_rectRipples.GetAt(i);
				if (IsValidReadPtr(pRect))
				{
					waterh = pRect->bottom-pRect->top;
					waterw = pRect->right-pRect->left;
					for (y = 0; y < waterh; y++)
					{
						p = (((double)(waterh-y))/((double)waterh));
		
						a = p * p * 48 + m_nIncv;
						yoff = y - (int)(sin(a) * 7) + 1;
	
						a = p * p * 64 + m_nInch;
						off = (int)(sin(a) * 10 * (1 - p));

						BitBlt(hBufDC, pRect->left+off, y+pRect->top, waterw, 1, m_hImageDC, pRect->left, pRect->top-yoff, SRCCOPY);
					}
				}
			}

			BitBlt(di.hdcDraw, 0, 0, m_nWidth, m_nHeight, hBufDC, 0, 0, SRCCOPY);
	
			DeleteDC(hBufDC);
			DeleteObject(hbmBuf);
		}

		return S_OK;
	}
	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		SetTimer(1, 33);

		return 0;
	}
	LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		InvalidateRect(NULL, FALSE);

		return 0;
	}
	LRESULT OnEraseBkgnd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = TRUE;

		return 0;
	}

	double	m_nIncv;
	double	m_nInch;
	int		m_nWidth;
	int		m_nHeight;
	HBITMAP m_hbmImage;
	HDC		m_hImageDC;
	CArrayPtr<CRect>	m_rectRipples;
};

#endif //__RIPPLE_H_
