// $Id: image.cpp,v 1.7 2000/02/23 02:06:13 mian Exp $

#include "stdafx.h"
#define JPEG_INTERNALS
#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>
#include "../common/util_safe.h"
extern "C"
{
#include "jpeglib/jpeglib.h"
#include "pnglib/png.h"
#include "tifflib/tiffio.h"
#include "tifflib/tif_msrc.h"
}

struct jpeg_custom_error_mgr 
{
	struct jpeg_error_mgr pub;
	jmp_buf setjmp_buffer;
};

struct png_data
{
	LPBYTE data;
};

struct pcx_header
{
	BYTE	id;
	BYTE	version;
	BYTE	encoding;
	BYTE	bits_per_pixel;
	SHORT	left_margin;
	SHORT	upper_margin;
	SHORT	right_margin;
	SHORT	lower_margin;
	SHORT	hres;
	SHORT	vres;
	RGBTRIPLE	palette[16];
	BYTE	reserved;
	BYTE	num_planes;
	SHORT	bytes_per_line;
	SHORT	palette_type;
	BYTE	filler[58];
};

struct pcx_data
{
	LPBYTE data;	
};

struct icon_header
{
	SHORT	reserved;
	SHORT	type;
	SHORT	count;
};

struct icon_entry
{
	BYTE	width;
	BYTE	height;
	BYTE	num_colors;
	BYTE	reserved;
	SHORT	num_planes;
	SHORT	bit_count;
	LONG	bytes;
	LONG	offset;
};

/*
	JPEG error handler
*/
void jpeg_error_handler(j_common_ptr cinfo)
{
	jpeg_custom_error_mgr *perr = (jpeg_custom_error_mgr *)cinfo->err;

	if (IsValidReadPtr(perr)) longjmp(perr->setjmp_buffer, 1);
}

/*
	Initialize JPEG data
*/
void jpeg_init_source(j_decompress_ptr cinfo)
{
}

/*
	Fill JPEG input buffer
*/
boolean jpeg_fill_input_buffer (j_decompress_ptr cinfo)
{
	static JOCTET jpeg_eoi[] = { 0xFF, JPEG_EOI };

	if (IsValidReadPtr(cinfo))
	{
		if (IsValidWritePtr(cinfo->src))
		{
			cinfo->src->next_input_byte = jpeg_eoi;
			cinfo->src->bytes_in_buffer = 2;
		}
	}

	return TRUE;
}

/*
	Skip data in JPEG buffer
*/
void jpeg_skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{

	if (IsValidReadPtr(cinfo))
	{
		if (IsValidWritePtr(cinfo->src))
		{
		  	cinfo->src->bytes_in_buffer -= num_bytes;
			cinfo->src->next_input_byte += num_bytes;

			if (cinfo->src->bytes_in_buffer < 0) jpeg_fill_input_buffer(cinfo);
		}
	}
}

/*
	Destroy JPEG buffer
*/
void jpeg_term_source(j_decompress_ptr cinfo)
{
}

/*
	Decode JPEG data specified in lpData and return a HBITMAP handle
*/
HANDLE LoadImageFromJPEG(LPBYTE lpData, DWORD dwFileSize)
{
	HBITMAP hbmImage = NULL;
	LPBYTE lpBits = NULL;
	jpeg_decompress_struct cinfo;
	jpeg_custom_error_mgr jerr;

	ZeroMemory(&cinfo, sizeof(cinfo));
	ZeroMemory(&jerr, sizeof(jerr));

	/*
		Create error handler
	*/
	cinfo.err = jpeg_std_error(&jerr.pub);
	jerr.pub.error_exit = jpeg_error_handler;

	/*
		If the JPEG handler triggers any errors, this will be called
	*/
	if (setjmp(jerr.setjmp_buffer))
	{
		jpeg_finish_decompress(&cinfo);
		jpeg_destroy_decompress(&cinfo);
		if (lpBits) free(lpBits);
		return NULL;
	}

	jpeg_create_decompress(&cinfo);

	/*
		Fill in callback functions
	*/
	cinfo.src = (struct jpeg_source_mgr *)cinfo.mem->alloc_small((j_common_ptr)&cinfo, JPOOL_PERMANENT, sizeof(struct jpeg_source_mgr));
	if (!IsValidWritePtr(cinfo.src)) longjmp(jerr.setjmp_buffer, 1);
	cinfo.src->next_input_byte = lpData;
	cinfo.src->bytes_in_buffer = dwFileSize;
	cinfo.src->init_source = jpeg_init_source;
	cinfo.src->fill_input_buffer = jpeg_fill_input_buffer;
	cinfo.src->skip_input_data = jpeg_skip_input_data;
	cinfo.src->resync_to_restart = jpeg_resync_to_restart;
	cinfo.src->term_source = jpeg_term_source;

	/*
		Fill in values from JPEG header
	*/
	jpeg_read_header(&cinfo, TRUE);
	cinfo.do_fancy_upsampling = TRUE;
	cinfo.dct_method = JDCT_FLOAT;

	/*
		Start decompressor
	*/
	jpeg_start_decompress(&cinfo);

	/*
		Allocate buffer to hold bitmap data
	*/
	LPBYTE row;
	JSAMPARRAY buffer = &row;
	lpBits = (LPBYTE)malloc(cinfo.image_height*cinfo.image_width*cinfo.output_components);
	if (!IsValidWritePtr(lpBits)) longjmp(jerr.setjmp_buffer, 1);

	unsigned int scanline = 0;
	while(scanline < cinfo.image_height)
	{
		/*
			Decode each scanline storing the decoded data in our bitmap buffer
		*/
		row = &lpBits[scanline*cinfo.image_width*cinfo.output_components];
		jpeg_read_scanlines(&cinfo, buffer, 1);
		scanline++;
	}

	/*
		Destroy JPEG objects
	*/
	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

	/*
		Assemble bitmap header+bitmap data into a HBITMAP
	*/
	BITMAP bm;
	bm.bmBitsPixel = cinfo.output_components*8;
	bm.bmHeight = cinfo.image_height;
	bm.bmWidth = cinfo.image_width;
	bm.bmPlanes = 1;
	bm.bmType = 0;
	bm.bmWidthBytes = cinfo.image_width*cinfo.output_components;
	bm.bmBits = lpBits;
	hbmImage = CreateBitmapIndirect(&bm);

	return hbmImage;
}

void png_user_error(png_structp png_ptr, png_const_charp error_msg)
{
}

void png_user_warning(png_structp png_ptr, png_const_charp warning_msg)
{
}

void png_read_data(png_structp png_ptr, png_bytep data, png_size_t length)
{
	png_data *p = (png_data *)png_get_io_ptr(png_ptr);

	if (IsValidReadPtr(p))
	{
		memcpy(data, p->data, length);
		p->data += length;
	}
}

/*
	Decode PNG data specified in lpData and return a HBITMAP handle
*/
HANDLE LoadImageFromPNG(LPBYTE lpData)
{
	HBITMAP hbmImage = NULL;
	png_structp png_ptr = NULL;
	png_infop png_info_ptr = NULL;
	png_uint_32 width = 0, height = 0;
	int bit_depth = 0, color_type = 0, interlace_type = 0;
	LPBYTE lpBits = NULL;
	png_data png_data;

	/*
		Create main PNG structures
	*/
    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, png_user_error, png_user_warning);
	png_info_ptr = png_create_info_struct(png_ptr);

	if (!IsValidReadPtr(png_ptr) || !IsValidReadPtr(png_info_ptr)) return NULL;

	/*
		Set the data feeder callback
	*/
	png_data.data = lpData;
	png_set_read_fn(png_ptr, &png_data, png_read_data);

	/*
		Read the PNG info
	*/
	png_read_info(png_ptr, png_info_ptr);

	/*
		Get some useful information about the image
	*/
	png_get_IHDR(png_ptr, png_info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, NULL, NULL);

	png_set_bgr(png_ptr);
	png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);

	/*
		Make PNG adjust the image if neccessary
	*/
	switch(color_type)
	{
		case PNG_COLOR_TYPE_GRAY_ALPHA:
		case PNG_COLOR_TYPE_GRAY:
			png_set_gray_to_rgb(png_ptr);
			png_set_expand(png_ptr);
			break;

		case PNG_COLOR_TYPE_PALETTE:
			png_set_expand(png_ptr);
			break;
	}

	/*
		Allocate row pointer memory
	*/
	png_bytep *row_pointers;
	row_pointers = (png_bytep *)malloc(height * sizeof(png_bytep *));
	if (row_pointers)
	{
		lpBits = (LPBYTE)malloc(height*width*4);

		if (lpBits)
		{
			/*
				Setup pointers to the beginning of each scanline
			*/
			unsigned int scanline;
			for (scanline = 0; scanline < height; scanline++)
			{
				row_pointers[scanline] = &lpBits[scanline*width*4];
			}

			/*
				Read in the image
			*/
			png_read_image(png_ptr, row_pointers);

			/*
				Cleanup the read process
			*/
			png_read_end(png_ptr, png_info_ptr);
		}

		free(row_pointers);
	}

	/*
		Destroy the main PNG structs
	*/
	png_destroy_read_struct(&png_ptr, &png_info_ptr, (png_infopp)NULL);

	/*
		Assemble bitmap header+bitmap data into a HBITMAP
	*/
	BITMAP bm;
	bm.bmBitsPixel = 32;
	bm.bmHeight = height;
	bm.bmWidth = width;
	bm.bmPlanes = 1;
	bm.bmType = 0;
	bm.bmWidthBytes = width*4;
	bm.bmBits = lpBits;
	hbmImage = CreateBitmapIndirect(&bm);

	return hbmImage;
}

/*
	Decode 8-bit PCX data and return a bitmap handle
	
	FIX: loads some, crashes on some
*/
HANDLE LoadImageFromPCX(LPBYTE lpData, DWORD dwFilesize)
{
	HBITMAP	hbmImage = NULL;
	struct pcx_header *pcx = (struct pcx_header *)lpData;
	int width = 0, height = 0, total_bytes = 0;
	LPBYTE lpBits = NULL;
	unsigned char *palette = NULL;
	unsigned char *pcx_data = NULL;
	jmp_buf setjmp_buffer;

	width = pcx->right_margin - pcx->left_margin + 1;
	height = pcx->lower_margin - pcx->upper_margin + 1;
	total_bytes = pcx->num_planes * pcx->bytes_per_line;

	/*
		Set error handler
	*/
	if (setjmp(setjmp_buffer))
	{
		if (lpBits) free(lpBits);
		if (pcx_data) free(pcx_data);
		return NULL;
	}

	/*
		Decode PCX data into a pixel buffer
	*/
	pcx_data = (unsigned char *)malloc(width*height);
	if (!IsValidWritePtr(pcx_data)) longjmp(setjmp_buffer, 1);
	int x, y, off;
	BYTE count, i, data;
	off=sizeof(struct pcx_header);
	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width;)
		{
			data = *(lpData+off);
			off++;
			/*
				Byte has runbit set
			*/
			if ((data & 0xC0) == 0xC0)
			{
				count = data & 0x3F;
				data = *(lpData+off);
				off++;
				for (i = 0; i < count; i++)
				{
					pcx_data[(y*width)+x] = data;
					x++;
				}
			} else {
				pcx_data[(y*width)+x] = data;
				x++;
			}
		}
		/*
			Clear any left over bytes
		*/
		for (;x < width; x++) off++;
	}

	if (*(lpData+(dwFilesize-769)) == 12) palette = lpData+(dwFilesize-768);

	/*
		Assemble PCX pixels into bitmap data
	*/
	lpBits = (LPBYTE)malloc(height*width*4);
	if (!IsValidWritePtr(lpBits)) longjmp(setjmp_buffer, 1);
	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width; x++)
		{
			/*
				Blue
			*/
			lpBits[(((y*width)+x)*4)] = palette[(pcx_data[(y*width)+x]*3)+2];
			/*
				Green
			*/
			lpBits[(((y*width)+x)*4)+1] = palette[(pcx_data[(y*width)+x]*3)+1];
			/*
				Red
			*/
			lpBits[(((y*width)+x)*4)+2] = palette[(pcx_data[(y*width)+x]*3)];
			/*
				Reserved
			*/
			lpBits[(((y*width)+x)*4)+3] = 0;
		}
	}
	free(pcx_data);

	/*
		Assemble bitmap header+bitmap data into a HBITMAP
	*/
	BITMAP bm;
	bm.bmBitsPixel = 32;
	bm.bmHeight = height;
	bm.bmWidth = width;
	bm.bmPlanes = 1;
	bm.bmType = 0;
	bm.bmWidthBytes = width*4;
	bm.bmBits = lpBits;
	hbmImage = CreateBitmapIndirect(&bm);

	return hbmImage;
}

HANDLE LoadImageFromTIF(LPBYTE lpData, DWORD dwFilesize)
{
	HBITMAP hbmImage = NULL;
	TIFFRGBAImage tiffImage;
	CHAR szError[256];
	LPBYTE lpTiffBits = NULL, lpBits = NULL;
	int width, height;

	TIFF* tif = TIFFOpenMem(lpData, dwFilesize);

	if (tif)
	{
		if (TIFFRGBAImageBegin(&tiffImage, tif, 0, szError))
		{

			width = tiffImage.width;
			height = tiffImage.height;

			/*
				Allocate RGBA memory
			*/
			lpTiffBits = (LPBYTE)malloc(height*width*4);
			if (IsValidWritePtr(lpTiffBits))
			{

				/*
					Get the tiff data as RGBA data
				*/
				TIFFRGBAImageGet(&tiffImage, (uint32 *)lpTiffBits, width, height);

				/*
					Correct the byte ordering
				*/
				lpBits = (LPBYTE)malloc(height*width*4);
				if (IsValidWritePtr(lpBits))
				{
					int x, y;
					/*
						Flip the image right side up
					*/
					for (y = 0; y < height; y++)
					{
						memcpy(lpBits+(y*width*4), lpTiffBits+((height-y-1)*width*4), width*4);
					}

					/*
						Convert RGBA to BGRA
					*/
					RGBQUAD pixel;
					for (y = 0; y < height; y++)
					{
						for (x = 0; x < width; x++)
						{
							pixel.rgbRed = lpBits[(((y*width)+x)*4)];
							pixel.rgbGreen = lpBits[(((y*width)+x)*4)+1];
							pixel.rgbBlue = lpBits[(((y*width)+x)*4)+2];
							pixel.rgbReserved = lpBits[(((y*width)+x)*4)+3];
							lpBits[(((y*width)+x)*4)] = pixel.rgbBlue;
							lpBits[(((y*width)+x)*4)+1] = pixel.rgbGreen;
							lpBits[(((y*width)+x)*4)+2] = pixel.rgbRed;
							lpBits[(((y*width)+x)*4)+3] = pixel.rgbReserved;
						}
					}

					free(lpTiffBits);

					/*
						Destroy the tiff image
					*/
					TIFFRGBAImageEnd(&tiffImage);

					/*
						Assemble bitmap header+bitmap data into a HBITMAP
					*/
					BITMAP bm;
					bm.bmBitsPixel = 32;
					bm.bmHeight = height;
					bm.bmWidth = width;
					bm.bmPlanes = 1;
					bm.bmType = 0;
					bm.bmWidthBytes = width*4;
					bm.bmBits = lpBits;
					hbmImage = CreateBitmapIndirect(&bm);
				}
			}
		}
		TIFFClose(tif);
	}

	return hbmImage;
}

HANDLE
WINAPI
LoadImageFromFileW(WCHAR *lpszPath)
{
	HANDLE hFile = NULL, hMapFile = NULL;
	LPBYTE lpData = NULL;
	HBITMAP hbmImage = NULL;

	/*
		Attempt to open file
	*/
	hFile = CreateFile(lpszPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (hFile != INVALID_HANDLE_VALUE)
	{
		hMapFile = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
		if (hMapFile != INVALID_HANDLE_VALUE)
		{
			/*
				Map file contents for easy access
			*/
			lpData = (LPBYTE)MapViewOfFile(hMapFile, FILE_MAP_READ, 0, 0, 0);
			if (IsValidReadPtr(lpData))
			{
				/*
					Attempt to detect file type
				*/
				if (*lpData == 0x42 && *(lpData+1) == 0x4D)
				{
					/*
						Image is a standard bitmap
					*/
					hbmImage = (HBITMAP)::LoadImage(NULL, lpszPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
				} else
				if (*lpData == 0xFF && *(lpData+1) == 0xD8 && *(lpData+2) == 0xFF)
				{
					/*
						Image is a JPEG
					*/
					hbmImage = (HBITMAP)::LoadImageFromJPEG(lpData, ::GetFileSize(hFile, NULL));
				} else
				if (*lpData == 0x89 && *(lpData+1) == 0x50 && *(lpData+2) == 0x4E && *(lpData+3) == 0x47)
				{
					/*
						Image is a PNG
					*/
					hbmImage = (HBITMAP)::LoadImageFromPNG(lpData);
				} else
				if (*lpData == 0x0A && *(lpData+1) == 0x05 && *(lpData+2) == 0x01 && *(lpData+3) == 0x08)
				{
					/*
						Image is a PCX
					*/
					hbmImage = (HBITMAP)::LoadImageFromPCX(lpData, ::GetFileSize(hFile, NULL));
				} else
				if (*(ULONG *)lpData == 0x002A4949 || *(ULONG *)lpData == 0x2A004D4D)
				{
					/*
						Image is a TIF
					*/
					hbmImage = (HBITMAP)::LoadImageFromTIF(lpData, ::GetFileSize(hFile, NULL));
				}
				UnmapViewOfFile(lpData);
			}
			CloseHandle(hMapFile);
		}
		CloseHandle(hFile);
	}

	return hbmImage;
}

HRGN
WINAPI
BitmapToRegion(HBITMAP hBmp, COLORREF cTransparentColor, COLORREF cTolerance, int xoffset, int yoffset)
{
	HRGN hRgn = NULL;

	if (hBmp)
	{
		// Create a memory DC inside which we will scan the bitmap content
		HDC hMemDC = CreateCompatibleDC(NULL);
		if (hMemDC)
		{
			// Get bitmap size
			HBITMAP hbm32;
			BITMAP bm;
			BITMAPINFOHEADER RGB32BITSBITMAPINFO;
			VOID * pbits32; 
			GetObject(hBmp, sizeof(bm), &bm);

			// Create a 32 bits depth bitmap and select it into the memory DC 
			RGB32BITSBITMAPINFO.biSize = sizeof(BITMAPINFOHEADER);
			RGB32BITSBITMAPINFO.biWidth = bm.bmWidth;
			RGB32BITSBITMAPINFO.biHeight = bm.bmHeight;
			RGB32BITSBITMAPINFO.biPlanes = 1;
			RGB32BITSBITMAPINFO.biBitCount = 32;
			RGB32BITSBITMAPINFO.biCompression = BI_RGB;
			RGB32BITSBITMAPINFO.biSizeImage = 0;
			RGB32BITSBITMAPINFO.biXPelsPerMeter = 0;
			RGB32BITSBITMAPINFO.biYPelsPerMeter = 0;
			RGB32BITSBITMAPINFO.biClrUsed = 0;
			RGB32BITSBITMAPINFO.biClrImportant = 0;

			hbm32 = CreateDIBSection(hMemDC, (BITMAPINFO *)&RGB32BITSBITMAPINFO, DIB_RGB_COLORS, &pbits32, NULL, 0);
			if (hbm32)
			{
				HBITMAP holdBmp = (HBITMAP)SelectObject(hMemDC, hbm32);

				// Create a DC just to copy the bitmap into the memory DC
				HDC hDC = CreateCompatibleDC(hMemDC);
				if (hDC)
				{
					// Get how many bytes per row we have for the bitmap bits (rounded up to 32 bits)
					HBITMAP holdBmp;
					BITMAP bm32;
					DWORD maxRects;
					HANDLE hData;
					RGNDATA *pData;
					HRGN h;
					BYTE *p32;
					int y, x;

					// Keep on hand highest and lowest values for the "transparent" pixels
					BYTE lr = GetRValue(cTransparentColor);
					BYTE lg = GetGValue(cTransparentColor);
					BYTE lb = GetBValue(cTransparentColor);
					BYTE hr = (BYTE) min(0xff, lr + GetRValue(cTolerance));
					BYTE hg = (BYTE) min(0xff, lg + GetGValue(cTolerance));
					BYTE hb = (BYTE) min(0xff, lb + GetBValue(cTolerance));

					GetObject(hbm32, sizeof(bm32), &bm32);
					while (bm32.bmWidthBytes % 4)
						bm32.bmWidthBytes++;

					// Copy the bitmap into the memory DC
					holdBmp = (HBITMAP)SelectObject(hDC, hBmp);
					BitBlt(hMemDC, 0, 0, bm.bmWidth, bm.bmHeight, hDC, 0, 0, SRCCOPY);

					// For better performances, we will use the ExtCreateRegion() function to create the
					// region. This function take a RGNDATA structure on entry. We will add rectangles by
					// amount of ALLOC_UNIT number in this structure.
					#define ALLOC_UNIT	100
					maxRects = ALLOC_UNIT;
					hData = GlobalAlloc(GMEM_MOVEABLE, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects));
					pData = (RGNDATA *)GlobalLock(hData);
					pData->rdh.dwSize = sizeof(RGNDATAHEADER);
					pData->rdh.iType = RDH_RECTANGLES;
					pData->rdh.nCount = pData->rdh.nRgnSize = 0;
					SetRect(&pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);

					// Scan each bitmap row from bottom to top (the bitmap is inverted vertically)
					p32 = (BYTE *)bm32.bmBits + (bm32.bmHeight - 1) * bm32.bmWidthBytes;
					for (y = 0; y < bm.bmHeight; y++)
					{
						// Scan each bitmap pixel from left to right
						for (x = 0; x < bm.bmWidth; x++)
						{
							// Search for a continuous range of "non transparent pixels"
							int x0 = x;
							LONG *p = (LONG *)p32 + x;
							while (x < bm.bmWidth)
							{
								BYTE b = GetRValue(*p);
								if (b >= lr && b <= hr)
								{
									b = GetGValue(*p);
									if (b >= lg && b <= hg)
									{
										b = GetBValue(*p);
										if (b >= lb && b <= hb)
											// This pixel is "transparent"
											break;
									}
								}
								p++;
								x++;
							}

							if (x > x0)
							{
								RECT *pr;

								// Add the pixels (x0, y) to (x, y+1) as a new rectangle in the region
								if (pData->rdh.nCount >= maxRects)
								{
									GlobalUnlock(hData);
									maxRects += ALLOC_UNIT;
									hData = GlobalReAlloc(hData, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), GMEM_MOVEABLE);
									pData = (RGNDATA *)GlobalLock(hData);
								}
								pr = (RECT *)&pData->Buffer;
								SetRect(&pr[pData->rdh.nCount], x0+xoffset, y+yoffset, x+xoffset, y+1+yoffset);
								if (x0 < pData->rdh.rcBound.left)
									pData->rdh.rcBound.left = x0+xoffset;
								if (y < pData->rdh.rcBound.top)
									pData->rdh.rcBound.top = y+yoffset;
								if (x > pData->rdh.rcBound.right)
									pData->rdh.rcBound.right = x+xoffset;
								if (y+1 > pData->rdh.rcBound.bottom)
									pData->rdh.rcBound.bottom = y+1+yoffset;
								pData->rdh.nCount++;

								// On Windows98, ExtCreateRegion() may fail if the number of rectangles is too
								// large (ie: > 4000). Therefore, we have to create the region by multiple steps.
								if (pData->rdh.nCount == 2000)
								{
									HRGN h = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData);
									if (hRgn)
									{
										CombineRgn(hRgn, hRgn, h, RGN_OR);
										DeleteObject(h);
									}
									else
										hRgn = h;
									pData->rdh.nCount = 0;
									SetRect(&pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);
								}
							}
						}

						// Go to next row (remember, the bitmap is inverted vertically)
						p32 -= bm32.bmWidthBytes;
					}

					// Create or extend the region with the remaining rectangles
					h = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData);
					if (hRgn)
					{
						CombineRgn(hRgn, hRgn, h, RGN_OR);
						DeleteObject(h);
					}
					else
						hRgn = h;

					// Clean up
					GlobalUnlock(hData);
					GlobalFree(hData);
					SelectObject(hDC, holdBmp);
					DeleteDC(hDC);
				}

				DeleteObject(SelectObject(hMemDC, holdBmp));
			}

			DeleteDC(hMemDC);
		}	
	}

	return hRgn;
}

/*
	$Log: image.cpp,v $
	Revision 1.7  2000/02/23 02:06:13  mian
	
	Added TIFF decoding the gutil
	Renamed Graphite Utility Library directory to gutil for easier access
	
	Revision 1.6  2000/02/22 12:26:00  mian
	
	Documented PCX decoder
	
	Revision 1.5  2000/02/22 11:40:57  mian
	
	Added PCX decoding
	
	Revision 1.4  2000/02/22 09:54:37  mian
	
	Made deskefx use LoadFromFromFile to support more image types
	Started on PCX image decoder
	
	Revision 1.3  2000/02/22 04:10:39  mian
	
	Better error checking in image routines
	
	Revision 1.2  2000/02/22 02:19:53  mian
	
	Cosmetic fixes
	
	Revision 1.1  2000/02/22 02:09:51  mian
	
	Added BitmapToRegion function
	
*/