// $Id: Graphite.cpp,v 1.15 2000/02/23 08:54:40 mian Exp $

#include "stdafx.h"

#include "../gutil/gutil32.h"

static TCHAR lpszGraphiteClass[] = L"GraphiteShellClass";

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
HRESULT LoadObjects();
HRESULT LoadScript(WCHAR *wszPath);

HINSTANCE		g_hInstance;
HWND			g_hWnd;
CScriptHost*	g_host;
CScriptObject*	g_objects;
IUnknown*		g_systray = NULL;

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	CoInitialize(NULL);

	g_hInstance = hInstance;

	/*
		Small hack to startup the Windows 2000 system tray objects, this will become
		user-definable in the startup object
	*/
	CLSID clsid;
	HRESULT hr = CLSIDFromString(L"{730F6CDC-2C86-11D2-8773-92E220524153}", &clsid);
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, IID_IUnknown, (void **)&g_systray);
	}

	WNDCLASS wc;
	ZeroMemory(&wc, sizeof(wc));
	wc.hInstance = g_hInstance;
	wc.lpszClassName = lpszGraphiteClass;
	wc.lpfnWndProc = WndProc;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	if (!RegisterClass(&wc))
	{
		MessageBox(NULL, L"Failed to register window class", L"Error", MB_OK);
		return 0;
	}

	g_hWnd = CreateWindowEx(
		WS_EX_TOPMOST,
		lpszGraphiteClass,
		NULL,
		WS_POPUP | WS_SYSMENU | WS_CAPTION,
		0, 0,
		50, 50,
		NULL,
		NULL,
		g_hInstance,
		NULL);

	if (!g_hWnd)
	{
		MessageBox(NULL, L"Failed to create window", L"Error", MB_OK);
		return 0;
	}
	ShowWindow(g_hWnd, SW_SHOW);

	/*
		Load scripting components
	*/
	g_objects = new CScriptObject;
	if (g_objects)
	{
		LoadObjects();
	}

	LoadScript(L"c:\\devel\\graphite\\theme.xml");

	/*
		TODO: Proper theme handling here, each script should run inside its own scripting host of the desired
			  scripting language.
	*/
	g_host = new CScriptHost;
	if (g_host)
	{
		g_host->AddRef();

		CLSID clsid;
		HRESULT hr;

		hr = CLSIDFromProgID(L"JScript", &clsid);
		if (SUCCEEDED(hr))
		{
			hr = g_host->CreateEngine(clsid);
			if (SUCCEEDED(hr))
			{
				/*
					Make our scripting objects visible to this scripting host
				*/
				g_objects->AddNamedItems(g_host);
/*				if (g_host->Run(L"g:\\test.js") != S_OK)
				{
					MessageBox(NULL, L"Failed to create scripting engine", L"Error", MB_OK);
				}
*/			}
		}
	}

	MSG msg;
	while (GetMessage(&msg, NULL, NULL, NULL))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	if (g_host)
	{
		g_host->Stop();
		g_host->Release();
	}

	if (g_objects)
	{
		delete g_objects;
		g_objects = NULL;
	}

	/*
		TODO: Remove temporary system tray hack
	*/
	if (g_systray) g_systray->Release();

	CoUninitialize();

	return 0;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_CLOSE:
			DestroyWindow(hWnd);
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

/*
	Load all the scripting components the user wants
*/
HRESULT LoadObjects()
{
	HKEY hKey, hSubKey;

	/*
		The components we wish to load are stored in HKCU\Software\Graphite\Components

		item -> CLSID/ProgID
	*/
	if (RegOpenKey(HKEY_CURRENT_USER, L"Software\\Graphite\\Components", &hKey) == ERROR_SUCCESS)
	{
		CLSID clsid;
		WCHAR wszItem[128], wszKey[256], wszValue[128];
		DWORD dwIndex, dwType, dwLength;
		HRESULT hr;
		for (dwIndex = 0; ; dwIndex++)
		{
			hr = RegEnumKey(hKey, dwIndex, wszItem, sizeof(wszItem));
			if (hr == ERROR_NO_MORE_ITEMS) break;
			if (hr == ERROR_SUCCESS)
			{
				wsprintf(wszKey, L"Software\\Graphite\\Components\\%s", wszItem);
				if (RegOpenKey(HKEY_CURRENT_USER, wszKey, &hSubKey) == ERROR_SUCCESS)
				{
					ZeroMemory(&wszValue, sizeof(wszValue));
					dwType = REG_SZ;
					dwLength = 128;
					if (RegQueryValueEx(hSubKey, NULL, NULL, &dwType, (LPBYTE)&wszValue, &dwLength) == ERROR_SUCCESS)
					{
						/*
							Assume string is a CLSID first
						*/
						hr = CLSIDFromString(wszValue, &clsid);
						if (SUCCEEDED(hr))
						{
							g_objects->LoadObject(wszItem, clsid);
						} else {
							/*
								Otherwise try for ProgID
							*/
							hr = CLSIDFromProgID(wszValue, &clsid);
							if (SUCCEEDED(hr))
							{
								g_objects->LoadObject(wszItem, clsid);
							} else {
								/*
									Failed both, must be invalid data or bad module
								Log("[Graphite] LoadObjects: Unable to load component: %s", wszValue);
								*/
							}
						}
					}
					RegCloseKey(hSubKey);
				}
			}
		}
		RegCloseKey(hKey);
	}

	return S_OK;
}

HRESULT ParseScript(MSXML::IXMLDOMNode* pXMLNode)
{
	HRESULT hr;
	BSTR bstrNodeName = NULL;
	MSXML::IXMLDOMNode* pXMLChild, *pXMLNext;

	hr = pXMLNode->get_nodeName(&bstrNodeName);
	if (SUCCEEDED(hr))
	{
		BSTR bstrText = NULL;
		/*
			Look for XML identifiers we need to parse
		*/
/*		if (!wcsicmp(bstrNodeName, L"title"))
		{
			hr = pXMLNode->get_text(&bstrText);
			if (SUCCEEDED(hr))
			{
				MessageBox(NULL, bstrText, L"Theme Title", MB_OK);
			}
		} else
		if (!wcsicmp(bstrNodeName, L"author"))
		{
			hr = pXMLNode->get_text(&bstrText);
			if (SUCCEEDED(hr))
			{
				MessageBox(NULL, bstrText, L"Theme Author", MB_OK);
			}
		} else
		if (!wcsicmp(bstrNodeName, L"description"))
		{
			hr = pXMLNode->get_text(&bstrText);
			if (SUCCEEDED(hr))
			{
				MessageBox(NULL, bstrText, L"Theme Description", MB_OK);
			}
		} else
*/		if (!wcsicmp(bstrNodeName, L"script"))
		{
			MSXML::IXMLDOMNamedNodeMap *pXMLAttrs = NULL;
			
			hr = pXMLNode->get_attributes(&pXMLAttrs);
			if (SUCCEEDED(hr))
			{
				pXMLAttrs->nextNode(&pXMLChild);
				while (pXMLChild)
				{
		            BSTR bstrValue = NULL;
					hr = pXMLChild->get_nodeName(&bstrValue);
					if (SUCCEEDED(hr))
					{
						if (!wcsicmp(bstrValue, L"language"))
						{
							VARIANT value;
							hr = pXMLChild->get_nodeValue(&value);
							if (SUCCEEDED(hr) && value.vt == VT_BSTR)
							{
//								MessageBox(NULL, V_BSTR(&value), L"Theme Language", MB_OK);
							}
							VariantClear(&value);
						}
					}
					SysFreeString(bstrValue);
		            pXMLChild->Release();
					pXMLAttrs->nextNode(&pXMLChild);
				}
			}
		}
		SysFreeString(bstrNodeName);
		SysFreeString(bstrText);
	}

	hr = pXMLNode->get_firstChild(&pXMLChild);
	while (pXMLChild)
	{
		ParseScript(pXMLChild);
		pXMLChild->get_nextSibling(&pXMLNext);
		pXMLChild->Release();
		pXMLChild = pXMLNext;
	}

	return hr;
}

HRESULT LoadScript(WCHAR *wszPath)
{
	HRESULT hr;
	MSXML::IXMLDOMDocument *pXMLDoc = NULL;
	MSXML::IXMLDOMNode* pXMLNode = NULL;

	hr = CoCreateInstance(MSXML::CLSID_DOMDocument, NULL, CLSCTX_INPROC_SERVER, MSXML::IID_IXMLDOMDocument, (void**)&pXMLDoc);
	if (SUCCEEDED(hr))
	{
		pXMLDoc->put_async(VARIANT_FALSE);

		VARIANT_BOOL vb;
		VARIANT vURL;
	    VariantInit(&vURL);
		vURL.vt = VT_BSTR;
		V_BSTR(&vURL) = wszPath;
		hr = pXMLDoc->load(vURL, &vb);
		if (SUCCEEDED(hr))
		{
			hr = pXMLDoc->QueryInterface(MSXML::IID_IXMLDOMNode, (void**)&pXMLNode);
			if (SUCCEEDED(hr))
			{
				ParseScript(pXMLNode);
				pXMLNode->Release();
			}
		}
		pXMLDoc->Release();
	}

	return hr;
}

/*
	$Log: Graphite.cpp,v $
	Revision 1.15  2000/02/23 08:54:40  mian
	
	Removed registry test
	
	Revision 1.14  2000/02/23 08:54:07  mian
	
	Added ability to execute registry startup items
	
	Revision 1.13  2000/02/23 02:06:13  mian
	
	Added TIFF decoding the gutil
	Renamed Graphite Utility Library directory to gutil for easier access
	
	Revision 1.12  2000/02/22 12:26:00  mian
	
	Documented PCX decoder
	
	Revision 1.11  2000/02/22 11:40:57  mian
	
	Added PCX decoding
	
	Revision 1.10  2000/02/22 09:54:37  mian
	
	Made deskefx use LoadFromFromFile to support more image types
	Started on PCX image decoder
	
	Revision 1.9  2000/02/22 02:09:51  mian
	
	Added BitmapToRegion function
	
	Revision 1.8  2000/02/17 15:11:42  mian
	
	Touched up XML theme
	
	Revision 1.7  2000/02/17 15:06:50  mian
	
	Started on XML theme parser and added default theme template
	
	Revision 1.6  2000/02/17 06:04:14  mian
	
	Fixed LoadObjects() not loading some objects, more shortcut item stuff
	
	Revision 1.5  2000/02/14 06:39:57  mian
	
	Changed component loading to only load those present in our registry key.
	Graphite Script Category still exists, we will use this to let the user choose
	what components they want enabled.  This way only we don't waste resources loading
	every component
	
	Revision 1.4  2000/02/14 03:23:34  mian
	
	more error checking in LoadObjects()
	
	Revision 1.3  2000/02/13 08:02:55  mian
	
	Added log messages
	
*/