// $Id: ScriptHost.cpp,v 1.3 2000/02/13 08:04:07 mian Exp $

#include "stdafx.h"
#include "ScriptHost.h"
#include "../common/util_safe.h"
#include "../common/util_strings.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern CScriptObject *g_objects;

CScriptHost::CScriptHost()
{
	m_refCount = 0;
	m_scrParse = NULL;
	m_scrEngine = NULL;
}

CScriptHost::~CScriptHost()
{
	if (IsValidInterfacePtr(m_scrParse))
	{
		m_scrParse->Release();
		m_scrParse = NULL;
	}
	if (m_scrEngine != NULL)
	{
		m_scrEngine->Release();
		m_scrEngine = NULL;
	}
}


// IUnknown
HRESULT STDMETHODCALLTYPE CScriptHost::QueryInterface(REFIID riid, void ** ppvObj)
{
	if (riid == IID_IUnknown)
	{
		*ppvObj = static_cast<IActiveScriptSite*>(this);
	}
	else if (riid == IID_IActiveScriptSite)
	{
		*ppvObj = static_cast<IActiveScriptSite*>(this);
	}
	else if (riid == IID_IActiveScriptSiteWindow)
	{
		*ppvObj = static_cast<IActiveScriptSiteWindow*>(this);
	}
	else
	{
		*ppvObj = NULL;
		return E_NOINTERFACE;
	}

	static_cast<IUnknown*>(*ppvObj)->AddRef();
	
	return S_OK;
}

ULONG STDMETHODCALLTYPE CScriptHost::AddRef()
{
	return ++m_refCount;
}

ULONG STDMETHODCALLTYPE CScriptHost::Release()
{
	if (--m_refCount == 0)
	{
		delete this;
		return 0;
	}
	return m_refCount;
}

// IActiveScriptSite

HRESULT STDMETHODCALLTYPE CScriptHost::GetLCID(/* [out] */ LCID __RPC_FAR *plcid)
{
	return E_NOTIMPL;
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::GetItemInfo(/* [in] */ LPCOLESTR pstrName,
									  /* [in] */ DWORD dwReturnMask,
								      /* [out] */ IUnknown __RPC_FAR *__RPC_FAR *ppiunkItem,
									  /* [out] */ ITypeInfo __RPC_FAR *__RPC_FAR *ppti)
{
	IUnknown *pUnk = NULL;
	g_objects->GetObjectUnknown((BSTR)pstrName, &pUnk);

	if (IsValidInterfacePtr(pUnk))
	{
		if (dwReturnMask & SCRIPTINFO_ITYPEINFO)
		{
			((IDispatch*)pUnk)->GetTypeInfo(0, NULL, ppti);
			(*ppti)->AddRef();
		}
		
		if (dwReturnMask & SCRIPTINFO_IUNKNOWN)
		{
			*ppiunkItem = (IDispatch*)pUnk;
			(*ppiunkItem)->AddRef();
		}
		return S_OK;
	}

	return E_INVALIDARG;
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::GetDocVersionString(/* [out] */ BSTR __RPC_FAR *pbstrVersion)
{
	return E_NOTIMPL;
}

        
HRESULT STDMETHODCALLTYPE CScriptHost::OnScriptTerminate(/* [in] */ const VARIANT __RPC_FAR *pvarResult,
											/* [in] */ const EXCEPINFO __RPC_FAR *pexcepinfo)
{
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::OnStateChange(/* [in] */ SCRIPTSTATE ssScriptState)
{
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::OnScriptError(/* [in] */ IActiveScriptError __RPC_FAR *pscripterror)
{

	EXCEPINFO	ei;
	DWORD		dwSrcContext;
	ULONG		ulLine;
	LONG		ichError;
	BSTR		bstrLine = NULL;
	WCHAR		wszError[1024];

	pscripterror->GetExceptionInfo(&ei);
	pscripterror->GetSourcePosition(&dwSrcContext, &ulLine, &ichError);
	pscripterror->GetSourceLineText(&bstrLine);

	wsprintf(wszError, L"%s\nSrc: %s\nLine:%d Error:%d Scode:%x", ei.bstrDescription, ei.bstrSource, ulLine, (int)ei.wCode, ei.scode);
	MessageBox(NULL, wszError, L"Script Error", MB_OK);

	return S_OK;
	
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::OnEnterScript( void)
{
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::OnLeaveScript( void)
{
	return S_OK;
}

// IActiveScriptSiteWindow

HRESULT STDMETHODCALLTYPE CScriptHost::GetWindow(/*[out]*/HWND __RPC_FAR *phwnd)
{
	phwnd = NULL;

	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CScriptHost::EnableModeless(/*[in]*/BOOL fEnable)
{
	return S_OK;
}

HRESULT CScriptHost::CreateEngine(REFCLSID clsid)
{
	HRESULT hr;

	hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, IID_IActiveScript, (void **)&m_scrEngine);
	if (FAILED(hr))
	{
		return hr;
	}

	hr = m_scrEngine->QueryInterface(IID_IActiveScriptParse, (void **)&m_scrParse);
	if (FAILED(hr))
	{
		m_scrEngine->Release();
		m_scrEngine = NULL;
		return hr;
	}

	hr = m_scrEngine->SetScriptSite(this);
	if (FAILED(hr))
	{
		m_scrParse->Release();
		m_scrParse = NULL;
		m_scrEngine->Release();
		m_scrEngine = NULL;
		return hr;
	}

	hr = m_scrParse->InitNew();
	if (FAILED(hr))
	{
		m_scrParse->Release();
		m_scrParse = NULL;
		m_scrEngine->Release();
		m_scrEngine = NULL;
		return hr;
	}

	return S_OK;
}

/*
	Run the specified script

	TODO: Re-write completely :)  Barfs on large scripts
*/
HRESULT CScriptHost::Run(BSTR bstrPath)
{
	HRESULT hr;
	BSTR bstrCode = NULL;
	HANDLE hFile;
	CHAR tszBuf[1024];
	DWORD dwBytesRead, dwTotalBytesRead = 0;

	hFile = CreateFile(bstrPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE)
	{
		ZeroMemory(&tszBuf, sizeof(tszBuf));
		while (ReadFile(hFile, tszBuf, sizeof(tszBuf), &dwBytesRead, NULL))
		{
			if (dwBytesRead == 0) break;
			tszBuf[dwBytesRead] = 0;
			CWStr wstr(tszBuf);
			if (bstrCode == NULL)
			{
				bstrCode = SysAllocStringLen(NULL, dwBytesRead);
				bstrCode[0] = 0;
			} else {
				SysReAllocStringLen(&bstrCode, NULL, dwTotalBytesRead+dwBytesRead);
			}
			wcscat(bstrCode, wstr.m_sz);
			dwTotalBytesRead += dwBytesRead;
		}
		CloseHandle(hFile);
		bstrCode[dwTotalBytesRead] = 0;
	}

	hr = m_scrParse->ParseScriptText(bstrCode, NULL, NULL, NULL, 0, 0, SCRIPTTEXT_ISVISIBLE, NULL, NULL);
	if (FAILED(hr))
	{
		return hr;
	}

	if (IsValidString(bstrCode)) SysFreeString(bstrCode);

	hr = m_scrEngine->SetScriptState(SCRIPTSTATE_CONNECTED);
	if (FAILED(hr))
	{
		return hr;
	}

	return S_OK;
}

HRESULT CScriptHost::Stop()
{
	if (IsValidInterfacePtr(m_scrEngine))
	{
		m_scrEngine->Close();
	}

	return S_OK;
}

HRESULT CScriptHost::AddItem(BSTR bstrItem)
{
	if (IsValidInterfacePtr(m_scrEngine))
	{
		return m_scrEngine->AddNamedItem(bstrItem, SCRIPTITEM_ISVISIBLE | SCRIPTITEM_ISSOURCE);
	}

	return E_FAIL;
}

/*
	$Log: ScriptHost.cpp,v $
	Revision 1.3  2000/02/13 08:04:07  mian
	
	Added log messages
	
*/